dist_canberra <- function(x){
  x[is.na(x)]<-0
  return(dist(x,method = "canberra"))
}

dist_manhattan <- function(x){
  x[is.na(x)]<-0
  return(dist(x,method = "manhattan"))
}

plot_references_lengths <- function(sp){
  its1 <- read.fasta(file.path("data/",paste(paste(sp,"sp",sep="")),paste(paste(sp,"sp",sep=""),"_ITS1.fasta", sep="")))
  its2 <- read.fasta(file.path("data/",paste(paste(sp,"sp",sep="")),paste(paste(sp,"sp",sep=""),"_ITS2.fasta", sep="")))
  a <- data.frame(getLength(its1))
  colnames(a) <- "Length"
  b <- data.frame(getLength(its2))
  colnames(b) <- "Length"
  
  p1 <- ggplot(a, aes(x=Length, fill=Length)) + 
    geom_histogram(color="black", fill="red", col="red", fill="green", alpha = .2, binwidth = 10) +
    #geom_histogram(data=b,fill = "blue", alpha = 0.2, color="black", fill="green", col="red", fill="green", alpha = .2, binwidth = 10) +
    scale_x_continuous(name = "ITS1 length", limits=c(0,1200)) +
    scale_y_continuous(name = "Count")
  
  p2 <- ggplot(b, aes(x=Length, fill=Length)) + 
    geom_histogram(color="black", fill="blue", col="red", fill="green", alpha = .2, binwidth = 10) +
    #geom_histogram(data=b,fill = "blue", alpha = 0.2, color="black", fill="green", col="red", fill="green", alpha = .2, binwidth = 10) +
    scale_x_continuous(name = "ITS2 length", limits=c(0,1200)) +
    scale_y_continuous(name = "Count")
  
  return(grid.arrange(p1,p2))
}


plot_depth <- function(seq.data){
  plot <- ggplot(seq.data, aes(x = tool, y = reads, fill = tool)) + 
    geom_bar(stat = "identity", position = "dodge") +
    geom_text(aes(label=reads), vjust=-0.25) +
    theme(axis.text.x = element_text(angle = 90), legend.position = "none") +
    scale_fill_manual(values=dedicated_palette) +
    scale_y_continuous(limits = c(0, max(seq.data$reads)+30000))
  return(plot)
}
# 
# build_phyloseq_objects <- function(metadata=metadata, path=path){
#   
#   # ALL
#   if(! file.exists(file.path(path,"all.rds"))){
#     # FROGS
#     if(! file.exists(file.path(path,"frogs.rds"))){
#       frogs <- create_phyloseq_object(file.path(path,"frogs.biom"), method="FROGS", metadata=metadata, taxMethod = "blast")
#       saveRDS(frogs,file.path(path,"frogs.rds"))
#     }else{
#       frogs <- readRDS(file.path(path,"frogs.rds"))
#       sample_data(frogs)$method <- "FROGS"
#       saveRDS(frogs,file.path(path,"frogs.rds"))
#     }
#     # DADA2
#     if(! file.exists(file.path(path,"dada2.rds"))){
#       dada2 <- create_phyloseq_object(file.path(path,"dada2.biom"), method="DADA2-pe", metadata=metadata, taxMethod = "blast")
#       saveRDS(dada2,file.path(path,"dada2.rds"))
#     }else{
#       dada2 <- readRDS(file.path(path,"dada2.rds"))
#       sample_data(dada2)$method <- "DADA2-pe"
#       saveRDS(dada2,file.path(path,"dada2.rds"))
#     }
#     # DADA2-se
#     if(! file.exists(file.path(path,"dada2_se.rds"))){
#       dada2_se <- create_phyloseq_object(file.path(path,"dada2-se.biom"), method="DADA2-se", metadata=metadata, taxMethod = "blast")
#       saveRDS(dada2_se,file.path(path,"dada2_se.rds"))
#     }else{
#       dada2_se <- readRDS(file.path(path,"dada2_se.rds"))
#       sample_data(dada2_se)$method <- "DADA2-se"
#       saveRDS(dada2_se,file.path(path,"dada2_se.rds"))
#     }
#     # EXPECTED
#     if(! file.exists(file.path(path,"real.rds"))){
#       real <- create_phyloseq_object(file.path(path,"real.biom"), method="EXPECTED", metadata=metadata, taxMethod = "blast")
#       #real <- transform_sample_counts(real, function(x) x+5000)
#       saveRDS(real,file.path(path,"real.rds"))
#     }else{
#       real <- readRDS(file.path(path,"real.rds"))
#       sample_data(real)$method <- "EXPECTED"
#       saveRDS(real,file.path(path,"real.rds"))
#     }
#     # QIIME-pe
#     if(! file.exists(file.path(path,"qiime_pe.rds"))){
#       qiime_pe <- create_phyloseq_object_from_qiime2(table=file.path(path,"pe-nonchimeric-wo-borderline-noSingle-table.qza"), taxtable=file.path(path,"pe-nonchimeric-wo-borderline-noSingle-tax.qza"), metadata=metadata, method="QIIME-pe")
#       saveRDS(qiime_pe,file.path(path,"qiime_pe.rds"))
#     }else{
#       qiime_pe <- readRDS(file.path(path,"qiime_pe.rds"))
#       sample_data(qiime_pe)$method <- "QIIME-pe"
#       saveRDS(qiime_pe,file.path(path,"qiime_pe.rds"))
#     }
#     # QIIME-se
#     if(! file.exists(file.path(path,"qiime_se.rds"))){
#       qiime_se <- create_phyloseq_object_from_qiime2(table=file.path(path,"se-nonchimeric-wo-borderline-noSingle-table.qza"), taxtable=file.path(path,"se-nonchimeric-wo-borderline-noSingle-tax.qza"), metadata=metadata, method="QIIME-se")
#       saveRDS(qiime_se,file.path(path,"qiime_se.rds"))
#     }else{
#       qiime_se <- readRDS(file.path(path,"qiime_se.rds"))
#       sample_data(qiime_se)$method <- "QIIME-se"
#       saveRDS(qiime_se,file.path(path,"qiime_se.rds"))
#     }
#     # USEARCH
#     if(! file.exists(file.path(path,"usearch.rds"))){  
#       usearch <- create_phyloseq_object_from_usearch(file.path(path,"usearch.biom"), method="USEARCH", metadata=metadata)
#       saveRDS(usearch,file.path(path,"usearch.rds"))
#     }else{
#       usearch <- readRDS(file.path(path,"usearch.rds"))
#       sample_data(usearch)$method <- "USEARCH"
#       saveRDS(usearch,file.path(path,"usearch.rds"))
#     }
#     all <- list("FROGS" = frogs, 
#                 "DADA2-pe" = dada2, 
#                 "DADA2-se" = dada2_se, 
#                 "EXPECTED" = real, 
#                 "QIIME-pe" = qiime_pe, 
#                 "QIIME-se" = qiime_se, 
#                 "USEARCH" = usearch)
#     saveRDS(all,file.path(path,"all.rds"))
#   }else{
#     all <- readRDS(file.path(path,"all.rds"))
#   }
#   return(all)
# }
# 
# build_global_object <- function(all){
#   data <- tibble(richeness = c(
#     nrow(tax_table(all$EXPECTED)),
#     nrow(tax_table(all$FROGS)), 
#     nrow(tax_table(all$USEARCH)),
#     nrow(tax_table(all$"DADA2-se")),
#     nrow(tax_table(all$"QIIME-se")),
#     nrow(tax_table(all$"DADA2-pe")),
#     nrow(tax_table(all$"QIIME-pe"))
#   ),
#   tool = c("EXPECTED", 
#            "FROGS", 
#            "USEARCH",
#            "DADA2-se",
#            "QIIME-se",
#            "DADA2-pe",
#            "QIIME-pe"),
#   reads = c(
#     sum(sample_sums(all$EXPECTED)),
#     sum(sample_sums(all$FROGS)), 
#     sum(sample_sums(all$USEARCH)),
#     sum(sample_sums(all$"DADA2-se")),
#     sum(sample_sums(all$"QIIME-se")),
#     sum(sample_sums(all$"DADA2-pe")),
#     sum(sample_sums(all$"QIIME-pe"))
#     
#   ),
#   physeq = all
#   )
#   # Order for graphics
#   data$tool <- factor(data$tool, levels=c("EXPECTED", 
#                                           "FROGS", 
#                                           "USEARCH",
#                                           "DADA2-se",
#                                           "QIIME-se",
#                                           "DADA2-pe",
#                                           "QIIME-pe"))
#   return(data)
# }

# plot_divergence <- function(dataset, divergence_file){
#   p1<-ggplot(as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset), aes(x=tool, y=Divergence, fill=tool)) +
#     geom_boxplot() + theme(plot.title = element_text(hjust = 0.5), plot.subtitle = element_text(hjust = 0.5), legend.position = "none") +
#     stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) +
#     scale_fill_manual(values=method_palette_wo_real) +
#     ggtitle("Divergence (%)", subtitle = dataset)
#   return(p1)
# }
# 
# plot_divergence <- function(dataset, divergence_file){
#   p1<-ggplot(as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset), aes(x=tool, color=tool, y=Divergence, fill=tool)) +
#     geom_boxplot() + theme(plot.title = element_text(hjust = 0.5), plot.subtitle = element_text(hjust = 0.5), legend.position = "none") +
#     stat_summary(fun = "median", geom = "point", aes(color = tool)) +
#     scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
#     stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) +
#     ggtitle("Divergence (%) at species level", subtitle = dataset)
#   return(p1)
# }

plot_divergence_real <- function(dataset, myits, divergence_file){
  data <- as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset) %>% filter(its == myits)
  data$tool <- factor(data$tool, levels=c("FROGS",
                                          "DADA2_FROGS", 
                                          "USEARCH",
                                          "DADA2-se",
                                          "QIIME-se",
                                          "DADA2-pe",
                                          "QIIME-pe"))
  p1<-ggplot(data, aes(x=tool, color=tool, y=Divergence, fill=tool)) +
    geom_boxplot() + theme(plot.title = element_text(hjust = 0.5), plot.subtitle = element_text(hjust = 0.5), legend.position = "none") +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) +
    ggtitle("Divergence (%) at species level", subtitle = dataset) #+
  #expand_limits(y = 0)
  return(p1)
}


# plot_metrics <- function(dataset, divergence_file){
#   p2<-ggplot(as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset), aes(x=tool, y=False.Negative, fill=tool)) +
#     geom_boxplot() +
#     stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) + 
#     scale_fill_manual(values=method_palette_wo_real) +
#     theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
#   p3<-ggplot(as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset), aes(x=tool, y=False.Positive, fill=tool)) +
#     geom_boxplot() +
#     stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" )  + 
#     scale_fill_manual(values=method_palette_wo_real) +
#     theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
#   p4<-ggplot(as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset), aes(x=tool, y=True.Positive, fill=tool)) +
#     geom_boxplot() +
#     stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) + 
#     scale_fill_manual(values=method_palette_wo_real) +
#     theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
#   #return(grid.arrange(p2, p3, p4, ncol=3, nrow = 1))
#   return(ggarrange(p2, p3, p4, ncol=3, nrow=1, common.legend = TRUE, legend="right"))
# }


# plot_metrics <- function(dataset, divergence_file){
#   p2<-ggplot(as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset), aes(x=tool, color=tool,y=False.Negative, fill=tool)) +
#     geom_boxplot() +
#     stat_summary(fun = "median", geom = "point", aes(color = tool)) +
#     scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
#     stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) + 
#     theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
#   p3<-ggplot(as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset), aes(x=tool, color=tool, y=False.Positive, fill=tool)) +
#     geom_boxplot() +
#     stat_summary(fun = "median", geom = "point", aes(color = tool)) +
#     scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
#     stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" )  + 
#     theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
#   p4<-ggplot(as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset), aes(x=tool, color=tool, y=True.Positive, fill=tool)) +
#     geom_boxplot() +
#     stat_summary(fun = "median", geom = "point", aes(color = tool)) +
#     scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
#     stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) + 
#     theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
#   p <- ggarrange(p2, p3, p4, ncol=3, nrow=1, common.legend = TRUE, legend="right")
#   p <- annotate_figure(p,
#                        top = text_grob("Metrics at species level")#,
#                        #fig.lab = "Figure 1", fig.lab.face = "bold"
#   )
#   return(p)
# }

plot_precision_and_recall <- function(dataset, divergence_file){
  infos <- as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset) %>% mutate(Precision = (True.Positive/(True.Positive + False.Positive))) %>% mutate(Recall.Rate = (True.Positive/(True.Positive + False.Negative)))
  p1<-ggplot(infos, aes(x=tool, color=tool, y=Recall.Rate, fill=tool)) +
    geom_boxplot() + theme(plot.title = element_text(hjust = 0.5), plot.subtitle = element_text(hjust = 0.5), legend.position = "none") +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    #stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) +
    theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
  p2<-ggplot(infos, aes(x=tool, color=tool, y=Precision, fill=tool)) +
    geom_boxplot() + theme(plot.title = element_text(hjust = 0.5), plot.subtitle = element_text(hjust = 0.5), legend.position = "none") +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    #stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) +
    theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
  return(ggarrange(p1, p2, ncol=2, nrow=1, common.legend = TRUE, legend="right"))
  
}

plot_metrics_real <- function(dataset, myits, divergence_file){
  
  
  data <- as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset) %>% filter(its == myits)
  data$tool <- factor(data$tool, levels=c("FROGS",
                                          "DADA2_FROGS", 
                                          "USEARCH",
                                          "DADA2-se",
                                          "QIIME-se",
                                          "DADA2-pe",
                                          "QIIME-pe"))
  
  p2<-ggplot(data, aes(x=tool, color=tool,y=False.Negative, fill=tool)) +
    geom_boxplot() +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) + 
    theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
  
  data <- as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset)
  data$tool <- factor(data$tool, levels=c("FROGS",
                                          "DADA2_FROGS", 
                                          "USEARCH",
                                          "DADA2-se",
                                          "QIIME-se",
                                          "DADA2-pe",
                                          "QIIME-pe"))
  
  p3<-ggplot(data, aes(x=tool, color=tool, y=False.Positive, fill=tool)) +
    geom_boxplot() +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" )  + 
    theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
  
  data <- as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset)
  data$tool <- factor(data$tool, levels=c("FROGS",
                                          "DADA2_FROGS", 
                                          "USEARCH",
                                          "DADA2-se",
                                          "QIIME-se",
                                          "DADA2-pe",
                                          "QIIME-pe"))
  
  p4<-ggplot(data, aes(x=tool, color=tool, y=True.Positive, fill=tool)) +
    geom_boxplot() +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) + 
    theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
  p <- ggarrange(p2, p3, p4, ncol=3, nrow=1, common.legend = TRUE, legend="right")
  p <- annotate_figure(p,
                       top = text_grob("Metrics at species level")#,
                       #fig.lab = "Figure 1", fig.lab.face = "bold"
  )
  return(p)
}

plot_precision_and_recall_real <- function(dataset, myits, divergence_file){
  infos <- as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset) %>% filter(its == myits) %>% mutate(Precision = (True.Positive/(True.Positive + False.Positive))) %>% mutate(Recall.Rate = (True.Positive/(True.Positive + False.Negative)))
  infos$tool <- factor(infos$tool, levels=c("FROGS",
                                            "DADA2_FROGS", 
                                            "USEARCH",
                                            "DADA2-se",
                                            "QIIME-se",
                                            "DADA2-pe",
                                            "QIIME-pe"))
  
  p1<-ggplot(infos, aes(x=tool, color=tool, y=Recall.Rate, fill=tool)) +
    geom_boxplot() + theme(plot.title = element_text(hjust = 0.5), plot.subtitle = element_text(hjust = 0.5), legend.position = "none") +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    #stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) +
    theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
  p2<-ggplot(infos, aes(x=tool, color=tool, y=Precision, fill=tool)) +
    geom_boxplot() + theme(plot.title = element_text(hjust = 0.5), plot.subtitle = element_text(hjust = 0.5), legend.position = "none") +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    #stat_compare_means(ref.group = "FROGS", method = "wilcox.test", label="p.signif" ) +
    theme(axis.title.x=element_blank(),axis.text.x=element_blank(),axis.ticks.x=element_blank())
  return(ggarrange(p1, p2, ncol=2, nrow=1, common.legend = TRUE, legend="right"))
  
}

create_phyloseq_object <- function(biom, method, metadata, taxMethod){
  physeq <- import_frogs(biom, taxMethod = taxMethod)
  sample_data(physeq) <- metadata
  sample_data(physeq)$method <- method
  #sample_data(physeq)$primer <- primer
  sample_names(physeq) <- glue::glue(paste("{sample_names(physeq)}_",method,sep=""))
  return(physeq)
}


# 
# create_phyloseq_object_from_qiime2 <- function(table=table, taxtable=taxtable, metadata=metadata, method=method){
#   SVs<-read_qza(table)
#   taxonomy<-read_qza(taxtable)
#   taxtable<-taxonomy$data %>% as.tibble() %>% separate(Taxon, sep=";", c("Kingdom","Phylum","Class","Order","Family","Genus","Species"))
#   physeq <- phyloseq(
#     otu_table(SVs$data, taxa_are_rows = T), 
#     tax_table(as.data.frame(taxtable) %>% select(-Confidence) %>% column_to_rownames("Feature.ID") %>% as.matrix()),
#     sample_data(metadata)
#   )
#   sample_names(physeq) <- glue::glue(paste("{sample_names(physeq)}_",method,sep=""))
#   sample_data(physeq)$method <- method
#   return(physeq)
# }
# 
# create_phyloseq_object_from_usearch <- function(biom, method=method, metadata=metadata){
#   physeq <- import_biom(biom)
#   sample_data(physeq) <- metadata
#   sample_data(physeq)$method <- method
#   sample_names(physeq) <- glue::glue(paste("{sample_names(physeq)}_",method,sep=""))
#   colnames(tax_table(physeq)) = c("Kingdom","Phylum","Class","Order","Family","Genus","Species")
#   return(physeq)
# }
# 
# merge_physeq_at_species <- function(physeq, references){
#   physeq_aglom <- phyloseq.extended::fast_tax_glom(physeq, taxrank = "Species")
#   physeq_aglom <- prune_taxa(tax_table(physeq_aglom)[,7] %in% references, physeq_aglom)
#   taxa_names(physeq_aglom) <- cbind(tax_table(physeq_aglom)[,7])
#   physeq_aglom_merged <- merge_samples(physeq_aglom, "method")
#   return(physeq_aglom_merged)
# }
# 
# update_frogs_physeq_with_multi <- function(physeq, multiaff_file){
#   
#   infos <- readr::read_tsv(file = multiaff_file) %>%
#     as_tibble() %>%
#     distinct(`#observation_name`, .keep_all = T) %>%
#     select(`#observation_name`, blast_taxonomy) %>%
#     separate(blast_taxonomy, ";(?=[\\S])", into = c("Kingdom","Phylum","Class","Order","Family","Genus","Species")) %>%
#     #mutate(across(Kingdom:Species, ~ stringr::str_remove(.x, ".:"))) %>%
#     tibble::column_to_rownames(var = "#observation_name") %>%
#     as("matrix")
#   
#   good <- tax_table(subset_taxa(physeq, Species != "Multi-affiliation" & Genus != "Multi-affiliation" & Family != "Multi-affiliation" & Order != "Multi-affiliation" & Class != "Multi-affiliation" & Phylum != "Multi-affiliation" & Kingdom != "Multi-affiliation" )) %>% as("matrix")
#   #na <- tax_table(subset_taxa(physeq, is.na(Species))) %>%  as("matrix")
#   all_infos <- unique(rbind(infos,good))#,na) # case where two species are identical but are multi-affiliated
#   tax_table(physeq) <- all_infos 
#   
#   return(physeq)
# }
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# 
# build_phyloseq_objects2 <- function(metadata=metadata, path=path){
#   
#   # ALL
#   if(! file.exists(file.path(path,"all.rds"))){
#     # FROGS
#     if(! file.exists(file.path(path,"frogs.rds"))){
#       frogs <- create_phyloseq_object(file.path(path,"frogs.biom"), method="frogs", metadata=metadata, taxMethod = "blast")
#       saveRDS(frogs,file.path(path,"frogs.rds"))
#     }else{
#       frogs <- readRDS(file.path(path,"frogs.rds"))
#     }
#     # DADA2
#     if(! file.exists(file.path(path,"dada2.rds"))){
#       dada2 <- create_phyloseq_object(file.path(path,"dada2.biom"), method="dada2", metadata=metadata, taxMethod = "blast")
#       saveRDS(dada2,file.path(path,"dada2.rds"))
#     }else{
#       dada2 <- readRDS(file.path(path,"dada2.rds"))
#     }
#     # DADA2-se
#     if(! file.exists(file.path(path,"dada2_se.rds"))){
#       dada2_se <- create_phyloseq_object(file.path(path,"dada2-se.biom"), method="dada2-se", metadata=metadata, taxMethod = "blast")
#       saveRDS(dada2_se,file.path(path,"dada2_se.rds"))
#     }else{
#       dada2_se <- readRDS(file.path(path,"dada2_se.rds"))
#     }
#     # EXPECTED
#     if(! file.exists(file.path(path,"real.rds"))){
#       real <- create_phyloseq_object(file.path(path,"real.biom"), method="real", metadata=metadata, taxMethod = "blast")
#       #real <- transform_sample_counts(real, function(x) x+5000)
#       saveRDS(real,file.path(path,"real.rds"))
#     }else{
#       real <- readRDS(file.path(path,"real.rds"))
#     }
#     # QIIME-pe
#     if(! file.exists(file.path(path,"qiime_pe.rds"))){
#       qiime_pe <- create_phyloseq_object_from_qiime2(table=file.path(path,"pe-nonchimeric-wo-borderline-noSingle-table.qza"), taxtable=file.path(path,"pe-nonchimeric-wo-borderline-noSingle-tax.qza"), metadata=metadata, method="qiime-pe")
#       saveRDS(qiime_pe,file.path(path,"qiime_pe.rds"))
#     }else{
#       qiime_pe <- readRDS(file.path(path,"qiime_pe.rds"))
#     }
#     # QIIME-se
#     if(! file.exists(file.path(path,"qiime_se.rds"))){
#       qiime_se <- create_phyloseq_object_from_qiime2(table=file.path(path,"se-nonchimeric-wo-borderline-noSingle-table.qza"), taxtable=file.path(path,"se-nonchimeric-wo-borderline-noSingle-tax.qza"), metadata=metadata, method="qiime-se")
#       saveRDS(qiime_se,file.path(path,"qiime_se.rds"))
#     }else{
#       qiime_se <- readRDS(file.path(path,"qiime_se.rds"))
#     }
#     # USEARCH
#     if(! file.exists(file.path(path,"usearch.rds"))){  
#       usearch <- create_phyloseq_object_from_usearch(file.path(path,"usearch.biom"), method="usearch", metadata=metadata)
#       saveRDS(usearch,file.path(path,"usearch.rds"))
#     }else{
#       usearch <- readRDS(file.path(path,"usearch.rds"))
#     }
#     all <- list("FROGS" = frogs, "DADA2-pe" = dada2, "DADA2-se" = dada2_se, "EXPECTED" = real, "QIIME-pe" = qiime_pe, "QIIME-se" = qiime_se, "USEARCH" = usearch)
#     saveRDS(all,file.path(path,"all.rds"))
#   }else{
#     all <- readRDS(file.path(path,"all.rds"))
#   }
#   return(all)
# }
# 


plot_richeness_graph <- function(data){
  p <- ggplot(data=data, aes(x=data$tool, y=data$richeness, fill=data$tool)) +
    xlab("Tool")  + ylab("Richness (OTU/ASV/ZOTU level)") +
    labs(fill = "Tool") +
    geom_bar(stat="identity")+theme_minimal() +
    geom_text(aes(label=data$richeness), vjust=-0.25) +
    theme(axis.text.x = element_text(angle = 90, hjust = 1), legend.position = "none") +
    scale_fill_manual(values=dedicated_palette) +
    scale_y_continuous(limits = c(0, max(data$richeness)+500))
  return(p)  
  #return(ggplotly(p, tooltip = NULL) %>%
  #         layout(xaxis = list(autorange = TRUE),
  #                yaxis = list(autorange = TRUE)))
}
# 
# plot_setmap <- function(mylist = mylist){
#   venn = Venn(mylist)
#   p <- setmap(venn,  element_clustering = TRUE)
#   return(p)
# }
# 
# plot_upset_graph <- function(mylist=mylist){
#   p <- upset(fromList(mylist), order.by = "freq", nsets = 7) #sets.bar.color = dedicated_palette)
#   return(p)
# }
# 
# # Not used
# plot_UpSet_graph <- function(mylist=mylist){
#   m <- make_comb_mat(mylist)
#   p <- UpSet(m, pt_size = unit(5, "mm"), lwd = 2, bg_col = c("red","green","green","green","green"),
#              comb_col = c("red", "blue", "black", "green", "green", "green", "green", "green")[comb_degree(m)])
#   return(p)
# }

build_heatmap_abundance <- function(mat, row_order, long_species_number){
  
  min <- min(replace_na(mat,0))
  max <- max(replace_na(mat,0))
  mat[mat==0] <- NA
  cluster_method <- dist_canberra(t(mat)) %>% hclust(method = "single")
  
  #row_order <- sort(rownames(mat),decreasing = FALSE)
  column_order <- dist_manhattan(t(mat)) %>% as.matrix() %>% `[`("EXPECTED",) %>% sort() %>% names()
  mat <- mat[row_order,]
  
  row_ha = rowAnnotation(foo = anno_mark(at = seq((nrow(mat)-long_species_number+1),nrow(mat)), labels = rep("long",long_species_number)))
  
  ha_top_annotation = HeatmapAnnotation(
    method = anno_text(cluster_method$labels, just = "right"),
    tool   = cluster_method$labels,
    col = list(tool = method_palette), 
    show_legend = FALSE
  )
  ha_bottom_annotation = HeatmapAnnotation(
    tool   = cluster_method$labels,
    col = list(tool = method_palette), 
    show_legend = FALSE
  )
  lgd_list = list(
    Legend(at = 1, title = "", labels=c("Absent"), legend_gp = gpar(fill=absent_color))
  )
  
  h <- Heatmap(mat, 
               # global
               na_col = absent_color, 
               height = 10,
               rect_gp = gpar(col= "black"), 
               name="Abundance", 
               border = TRUE, 
               col = colorRamp2(seq(min, max, length = 2), c("#e6eded", "#335e61")), 
               # rows
               cluster_rows = FALSE, 
               row_title= "Species", 
               row_order = row_order, 
               show_row_dend = FALSE, 
               row_names_side = "left", 
               # columns
               column_dend_side = "top", 
               column_dend_height = unit(20, "mm"), 
               cluster_columns = cluster_method,
               column_order = column_order,
               # annotations
               right_annotation = row_ha, 
               top_annotation = ha_top_annotation, 
               bottom_annotation = ha_bottom_annotation
  )
  
  return(list(h, mat, lgd_list))
}

build_heatmap_presence <- function(mat, row_order, long_species_number){
  
  cluster_method <- dist_manhattan(t(mat)) %>% hclust(method = "single")
  #rownames(mat) <- rownames(mat) %>% as_tibble() %>% separate(value, ";(?=[\\S])", into = c("Kingdom","Phylum","Class","Order","Family","Genus","Species"), remove=FALSE) %>% select(Species) %>% pull()
  #row_order <- sort(rownames(mat),decreasing = FALSE)
  #mat <- mat[row_order,]
  
  row_ha = rowAnnotation(foo = anno_mark(at = seq((nrow(mat)-long_species_number+1),nrow(mat)), labels = rep("long",long_species_number)))
  
  ha_top_annotation = HeatmapAnnotation(
    method = anno_text(colnames(mat)),
    tool   = colnames(mat),
    col = list(tool = method_palette), 
    show_legend = FALSE
  )
  ha_bottom_annotation = HeatmapAnnotation(
    tool   = colnames(mat),
    col = list(tool = method_palette), 
    show_legend = FALSE
  ) 
  
  colors = structure(c(absent_color,"#b1c5c4","#86a5a4"), names = c(0,0.75,1))
  column_order <- dist_manhattan(t(mat)) %>% as.matrix() %>% `[`("EXPECTED",) %>% sort() %>% names()
  
  h <- Heatmap(mat, 
               # Global
               rect_gp = gpar(col= "black"),
               col=colors,
               border = TRUE, 
               #name="Abundance", 
               # Rows
               cluster_rows = FALSE,  
               #row_title= "Species",
               row_order = row_order,
               show_row_dend = TRUE, 
               row_names_side = "left",  
               # Columns
               column_title = "manhattan distance",
               cluster_columns = FALSE, 
               column_order = column_order, 
               # Annotations and legends
               heatmap_legend_param = list(title = "Seed sequence", labels=c("Absent","Not well reconstructed","Perfect")), 
               top_annotation = ha_top_annotation, 
               right_annotation = row_ha, 
               bottom_annotation = ha_bottom_annotation
  )
  
  return(list(h, mat))
}


fill_abundance_matrix <- function(mat, tsv_infos, tool, multiaff_infos){
  abundance_total <- sum(tsv_infos$observation_sum)
  
  for(taxo in unique(tsv_infos$blast_taxonomy)){
    new_value_abundance <- tsv_infos %>% filter(blast_taxonomy == taxo) %>% select(observation_sum) %>% sum()
    new_value_percent_abundance <- new_value_abundance/abundance_total
    if(taxo %in% rownames(mat)){
      mat[taxo,tool] <- mat[taxo,tool]+new_value_percent_abundance
    }else{
      if(!(tool=="FROGS" | tool=="DADA2_FROGS")){
        mat["Others",tool] <- mat["Others",tool]+new_value_percent_abundance  
      }
    }
  }
  if(!missing(multiaff_infos)){
    for (cluster in unique(multiaff_infos$"#observation_name")) {
      nb_multi_by_cluster <- as.integer(table(multiaff_infos$"#observation_name")[cluster])
      new_value_abundance <- as.integer(tsv_infos %>% filter(observation_name == cluster) %>% select(observation_sum) %>% pull())/nb_multi_by_cluster
      new_value_percent_abundace <- new_value_abundance/abundance_total
      for(taxo in multiaff_infos %>% filter(`#observation_name` == cluster) %>% select(blast_taxonomy) %>% pull()){
        if(taxo %in% rownames(mat)){
          mat[taxo,tool] <- mat[taxo,tool]+new_value_percent_abundance  
        }
        else{
          mat["Others",tool] <- mat["Others",tool]+new_value_percent_abundance  
        }
        
      }
    }
  }
  return(mat)
}


fill_presence_matrix <- function(mat, real_infos, tsv_infos, tool, multiaff_infos){
  
  for (cluster in unique(tsv_infos$"observation_name")) {
    
    taxo <- as.character(tsv_infos %>% filter(observation_name == cluster) %>% select(blast_taxonomy) %>% pull())
    
    expected_length <- real_infos %>% filter(Taxonomy == taxo) %>% select(Sequence) %>% pull() %>% str_length()
    length <- tsv_infos %>% filter(observation_name == cluster) %>% select(seed_sequence) %>% pull() %>% str_length()
    expected_seq <- real_infos %>% filter(Taxonomy == taxo) %>% select(Sequence) %>% pull()
    otu_seq <- tsv_infos %>% filter(observation_name == cluster) %>% select(seed_sequence) %>% pull()
    
    if(taxo %in% rownames(mat)){
      if(grepl("FROGS_combined", cluster)){
        spl <- str_split(otu_seq, "NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN")
        part1 <- spl[[1]][1]
        part2 <- spl[[1]][2]
        if(grepl(part1, expected_seq) && grepl(part2, expected_seq)){
          mat[taxo,tool] <- 1  
        }else{
          mat[taxo,tool] <- max(mat[taxo,tool],0.75)
        }
      }
      else{
        if(grepl( otu_seq, expected_seq, fixed = TRUE) && expected_length == length){
          mat[taxo,tool] <- 1
        }else{
          mat[taxo,tool] <- max(mat[taxo,tool],0.75) 
        }  
      }
    }
  }
  
  
  if(!missing(multiaff_infos)){
    for (cluster in unique(multiaff_infos$"#observation_name")) {
      taxo <- multiaff_infos %>% filter(observation_name == cluster) %>% select(blast_taxonomy) %>% pull()
      for(multi in taxo){
        expected_length <- real_infos %>% filter(Taxonomy == multi) %>% pull() %>% str_length()
        length <- tsv_infos %>% filter(observation_name == cluster) %>% select(seed_sequence) %>% pull() %>% str_length()
        expected_seq <- real_infos %>% filter(Taxonomy == multi) %>% select(Sequence) %>% pull()
        otu_seq <- tsv_infos %>% filter(observation_name == cluster) %>% select(seed_sequence) %>% pull()
        if(multi %in% rownames(mat)){
          if(grepl("FROGS_combined", cluster)){
            spl <- str_split(otu_seq, "NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN")
            part1 <- spl[[1]][1]
            part2 <- spl[[1]][2]
            if(grepl(part1, expected_seq) && grepl(part2, expected_seq)){
              mat[multi,tool] <- 1  
            }else{
              mat[multi,tool] <- max(mat[multi,tool],0.75)
            }
          }
          else{
            if(grepl( otu_seq, expected_seq, fixed = TRUE) && expected_length == length){
              mat[multi,tool] <- 1  
            }else{
              mat[multi,tool] <- max(mat[multi,tool],0.75) 
            }
          }
        }
      }
    }
  }
  return(mat)
}


build_heatmap_abundance_real <- function(mat){
  
  min <- min(replace_na(mat,0))
  max <- max(replace_na(mat,0))
  mat[mat==0] <- NA
  cluster_method <- dist_canberra(t(mat)) %>% hclust(method = "single")
  
  row_order <- sort(rownames(mat),decreasing = FALSE)
  column_order <- dist_manhattan(t(mat)) %>% as.matrix() %>% `[`("EXPECTED",) %>% sort() %>% names()
  mat <- mat[row_order,]
  
  ha_top_annotation = HeatmapAnnotation(
    method = anno_text(cluster_method$labels, just = "right"),
    tool   = cluster_method$labels,
    col = list(tool = method_palette), 
    show_legend = FALSE
  )
  ha_bottom_annotation = HeatmapAnnotation(
    tool   = cluster_method$labels,
    col = list(tool = method_palette), 
    show_legend = FALSE
  )
  lgd_list = list(
    Legend(at = 1, title = "", labels=c("Absent"), legend_gp = gpar(fill=absent_color))
  )
  
  h <- Heatmap(mat, 
               # global
               na_col = absent_color, 
               height = 10,
               rect_gp = gpar(col= "black"), 
               name="Abundance", 
               border = TRUE, 
               col = colorRamp2(seq(min, max, length = 2), c("#e6eded", "#335e61")), 
               # rows
               cluster_rows = FALSE, 
               row_title= "Species", 
               row_order = row_order, 
               show_row_dend = FALSE, 
               row_names_side = "left", 
               # columns
               column_dend_side = "top", 
               column_dend_height = unit(20, "mm"), 
               cluster_columns = cluster_method,
               column_order = column_order,
               # annotations
               #right_annotation = row_ha, 
               top_annotation = ha_top_annotation, 
               bottom_annotation = ha_bottom_annotation
  )
  
  return(list(h, mat, lgd_list))
}

build_heatmap_presence_real <- function(mat){
  
  cluster_method <- dist_manhattan(t(mat)) %>% hclust(method = "single")
  #rownames(mat) <- rownames(mat) %>% as_tibble() %>% separate(value, ";(?=[\\S])", into = c("Kingdom","Phylum","Class","Order","Family","Genus","Species"), remove=FALSE) %>% select(Species) %>% pull()
  row_order <- sort(rownames(mat),decreasing = FALSE)
  mat <- mat[row_order,]
  
  
  
  ha_top_annotation = HeatmapAnnotation(
    method = anno_text(colnames(mat)),
    tool   = colnames(mat),
    col = list(tool = method_palette), 
    show_legend = FALSE
  )
  ha_bottom_annotation = HeatmapAnnotation(
    tool   = colnames(mat),
    col = list(tool = method_palette), 
    show_legend = FALSE
  ) 
  
  colors = structure(c(absent_color,"#b1c5c4","#86a5a4"), names = c(0.00,0.75,1.00))
  column_order <- dist_manhattan(t(mat)) %>% as.matrix() %>% `[`("EXPECTED",) %>% sort() %>% names()
  
  h <- Heatmap(mat, 
               # Global
               rect_gp = gpar(col= "black"),
               col=colors,
               border = TRUE, 
               #name="Abundance", 
               # Rows
               cluster_rows = FALSE,  
               #row_title= "Species",
               row_order = row_order,
               show_row_dend = TRUE, 
               row_names_side = "left",  
               # Columns
               column_title = "manhattan distance",
               cluster_columns = FALSE, 
               column_order = column_order, 
               # Annotations and legends
               heatmap_legend_param = list(title = "Seed sequence", at = c(0,0.75,1), labels=c("Absent","Not well reconstructed","Perfect")), 
               top_annotation = ha_top_annotation, 
               bottom_annotation = ha_bottom_annotation
  )
  
  return(list(h, mat))
}


plot_precision <- function(dataset, divergence_file){
  infos <- as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset) %>% mutate(Precision = (True.Positive/(True.Positive + False.Positive))) %>% mutate(Recall.Rate = (True.Positive/(True.Positive + False.Negative)))
  pp<-ggplot(infos, aes(x=tool, color=tool, y=Precision, fill=tool)) +
    geom_boxplot() + theme(axis.text.x=element_text(size=13), axis.title.y=element_text(size=13), plot.title=element_text(size=18, hjust = 0.5), plot.subtitle = element_text(size=14, hjust = 0.5), legend.position = "none") +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    ggtitle("Precision at species level", subtitle = paste(strsp,its,law, sep="-"))
  
  return(pp)
  
}

plot_recall <- function(dataset, divergence_file){
  infos <- as_tibble(read.table(divergence_file, sep="\t", header=TRUE)) %>% filter(data == dataset) %>% mutate(Precision = (True.Positive/(True.Positive + False.Positive))) %>% mutate(Recall.Rate = (True.Positive/(True.Positive + False.Negative)))
  
  pr<-ggplot(infos, aes(x=tool, color=tool, y=Recall.Rate, fill=tool)) +
    geom_boxplot() + theme(axis.text.x=element_text(size=13), axis.title.y=element_text(size=13), plot.title=element_text(size=18, hjust = 0.5), plot.subtitle = element_text(size=14, hjust = 0.5), legend.position = "none") +
    stat_summary(fun = "median", geom = "point", aes(color = tool)) +
    scale_fill_manual(values=method_palette_wo_real, aesthetics = c("colour", "fill")) +
    ggtitle("Recall rate at species level", subtitle = paste(strsp,its,law, sep="-"))
  return(pr)
  
}


build_heatmap_one_tool <- function(mat, min, max){
  row_order <- sort(rownames(mat),decreasing = FALSE)
  h <- Heatmap(mat, 
               # global
               na_col = absent_color, 
               height = 10,
               rect_gp = gpar(col= "black"), 
               name="Abundance", 
               border = TRUE, 
               col = colorRamp2(seq(min, max, length = 2), c("#e6eded", "#335e61")), 
               # rows
               cluster_rows = FALSE, 
               #row_title= "Species", 
               row_order = row_order, 
               show_row_dend = FALSE, 
               row_names_side = "left", 
               # columns
               column_dend_side = "top", 
               #column_title = column_title,
               column_dend_height = unit(20, "mm")
               #cluster_columns = cluster_method,
               #column_order = column_order,
               # annotations
               #right_annotation = row_ha, 
               #top_annotation = ha_top_annotation, 
               #bottom_annotation = ha_bottom_annotation
  )
  return(h)
}

build_heatmap_one_tool_presence <- function(mat, min, max){
  row_order <- sort(rownames(mat),decreasing = FALSE)
  colors = structure(c(absent_color,partially_color,perfect_color), names = c(0,0.75,1))
  h <- Heatmap(mat, 
               # global
               na_col = absent_color, 
               height = 10,
               rect_gp = gpar(col= "black"), 
               #name="Abundance", 
               border = TRUE, 
               col = colors,
               # rows
               cluster_rows = FALSE, 
               #row_title= "Species", 
               row_order = row_order, 
               show_row_dend = FALSE, 
               row_names_side = "left", 
               # columns
               column_dend_side = "top", 
               #column_title = column_title,
               column_dend_height = unit(20, "mm"),
               show_heatmap_legend = FALSE
               #heatmap_legend_param = list(title = "Seed sequence", labels=c("Absent","Bad length","Perfect"))
               #cluster_columns = cluster_method,
               #column_order = column_order,
               # annotations
               #right_annotation = row_ha, 
               #top_annotation = ha_top_annotation, 
               #bottom_annotation = ha_bottom_annotation
  )
  return(h)
}

build_heatmap_one_tool_presence_amplicons <- function(mat, min, max, title){
  col_order <- sort(colnames(mat),decreasing = FALSE)
  colors = structure(c(absent_color,partially_color,perfect_color), names = c(0,0.75,1))
  h <- Heatmap(mat, 
               #use_raster = TRUE, raster_quality = 5,
               column_title = title,
               # global
               na_col = absent_color, 
               height = 10,
               rect_gp = gpar(col= "black"), 
               #name="Abundance", 
               border = FALSE, 
               col = colors,
               # rows
               cluster_rows = FALSE, 
               #row_title= "Species", 
               column_order = col_order, 
               column_title_gp = gpar(fontsize = 10),
               show_row_dend = FALSE, 
               show_column_names = FALSE,
               row_names_side = "left", 
               row_names_gp = gpar(fontsize = 8),
               # columns
               column_dend_side = "top", 
               #column_title = column_title,
               column_dend_height = unit(20, "mm"),
               show_heatmap_legend = FALSE
               #heatmap_legend_param = list(title = "Seed sequence", labels=c("Absent","Bad length","Perfect"))
               #cluster_columns = cluster_method,
               #column_order = column_order,
               # annotations
               #right_annotation = row_ha, 
               #top_annotation = ha_top_annotation, 
               #bottom_annotation = ha_bottom_annotation
  )
  return(h)
}

ggplotRegression <- function (fit, eco, marker,type) {
  ggplot(fit$model, aes_string(x = names(fit$model)[2], y = names(fit$model)[1])) + 
    geom_point() +
    theme(plot.title = element_text(size = 8)) +
    stat_smooth(method = "lm", col = "red") +
    labs(title = paste(eco,
                       marker,
                       #type,
                       "Adj R2 = ",signif(summary(fit)$adj.r.squared, 5),
                       "Intercept =",signif(fit$coef[[1]],5 ),
                       " Slope =",signif(fit$coef[[2]], 5),
                       " P =",signif(summary(fit)$coef[2,4], 5)))
}


